#include <stdio.h>
#include <string.h>
#include <usb.h>

#include <sys/stat.h>
#include <limits.h>

#include "ifp_routines.h"
#define INFO "!!!INFO!!!"

#define IFP_REQ_TYPE			0xc0
#define IFP_GET_CAPACITY_REQUEST	0x14
#define IFP_GET_FREE_REQUEST		0x15
#define IFP_SEND_DATA			0x18
#define IFP_LS				0x0f
#define IFP_LS_NEXT			0x10
#define IFP_BULK_TO			0x03
#define IFP_BULK_FROM			0x83
#define IFP_LS_LEAVE			0x11
#define IFP_GET_FILE_INFO_SIZE		0x0b
#define IFP_FILE_INFO			0x05
#define IFP_FILE_LEAVE			0x0d
#define IFP_FILE_UPLOAD			0x06
#define IFP_FILE_DOWNLOAD		0x07
#define IFP_FILE_UPLOAD_APPEND		0x08
#define IFP_FILE_DELETE			0x0e
#define IFP_DIR_DELETE			0x13
#define IFP_DIR_CREATE			0x12
#define IFP_FORMAT			0x16
#define IFP_FIRMWARE_UPDATE		0x17
#define IFP_02_COMMAND			0x02
#define IFP_02_BATTERY			0x08
#define IFP_02_STRING			0x00
#define IFP_02_FIRMWARE			0x03

#define TOUT 1000

#define IFP_BULK_MAXPATHLEN		0x400
#define IFP_MAXPATHLEN			0x200


int new_list_dir(usb_dev_handle *dh, const char *name, int format, int recurrent)
{
  unsigned char ctl[4];
  unsigned char ctl8[8];
  unsigned char inbuf[0x100];
  unsigned char strbuf[0x200];
  unsigned char strbuf2[0x200];
  unsigned char strnew[0x200];
  int ret, size;
  char *prefix = name;

  fprintf(stderr, "Starting work...name %s \n", name);
  if (ifp_send_filename(dh, name) < 0) 
    return -1;

  usb_control_msg(dh, IFP_REQ_TYPE, IFP_LS, 0, 0, ctl, sizeof(ctl), TOUT);
  fprintf(stderr, "ctl: %d%d%d%d\n", ctl[0], ctl[1], ctl[2], ctl[3]);
  if (!ifp_ok(ctl)) return -1;

  while (1) {
    usb_control_msg(dh, IFP_REQ_TYPE, IFP_LS_NEXT, 3, 0, ctl, sizeof(ctl), TOUT);
    ret = ifp_ctl_ret(ctl);
    switch(ret) {
      case 0:
        goto LS_END;
      case 1: /* Normal File */
				usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
				unicode2string(strbuf,inbuf);
				strcpy(strbuf2,name);
				if (strcmp(name,"/") != 0) strcat(strbuf2,"/");
				strcat(strbuf2,strbuf); // path/file
				ifp_send_filename(dh,strbuf2);
				usb_control_msg(dh,IFP_REQ_TYPE,IFP_FILE_INFO,1,0,ctl,sizeof(ctl),TOUT);
				if (!ifp_ok(ctl))
					fprintf(stderr, "error\n");
				usb_control_msg(dh,IFP_REQ_TYPE,IFP_GET_FILE_INFO_SIZE,0,0,ctl8,sizeof(ctl8),TOUT);
				//printf("ctl8: %d %d %d %d %d %d %d %d\n",ctl8[0],ctl8[1],ctl8[2],ctl8[3],ctl8[4],ctl8[5],ctl8[6],ctl8[7],ctl8[8]);
				size = ctl8[4] + ctl8[5]*256 + ctl8[6]*256*256 + ctl8[7]*256*256*256;
				if (format)
					printf("-rw-rw-rw-   1 root     root  %d Jan  1 1980 %s/%s\n",size,prefix,strbuf);
				else 
					printf("f %s\t(size %d)\n",strbuf,size);
				usb_control_msg(dh,IFP_REQ_TYPE,IFP_FILE_LEAVE,0,0,ctl,sizeof(ctl),TOUT); //??????
       break;
      case 2: /* Directory */
        usb_bulk_read(dh, IFP_BULK_FROM, inbuf, sizeof(inbuf), TOUT);
        fprintf(stderr, "inbuf : %s\n", inbuf);
        unicode2string(strbuf, inbuf);
        fprintf(stderr, " %s/%s\n", prefix, strbuf);
        break;
      default:
        fprintf(stderr, "Uknown LS_NEXT return value = %d\n", ret);
        return -3;
    }
  }
LS_END:
  usb_control_msg(dh, IFP_REQ_TYPE, IFP_LS_LEAVE, 0, 0, ctl, sizeof(ctl), TOUT);

  return 0;
}

int ifp_list_dir_printf2(usb_dev_handle *dh, const char *name, int format) {
    unsigned char ctl[4];
    unsigned char ctl8[8];
    unsigned char inbuf[0x100];
    unsigned char strbuf[0x200];
    unsigned char strbuf2[0x200];
    unsigned char strnew[0x200];
    char *dir;
    int ret;
    int size;
    const char *prefix = (strcmp(name,"/") == 0) ? (name+1) : name;

    if (ifp_send_filename(dh,name) < 0) return -1;
    
    usb_control_msg(dh,IFP_REQ_TYPE,IFP_LS,0,0,ctl,sizeof(ctl),TOUT);
    if (!ifp_ok(ctl)) return -1;
    while (1) {
			usb_control_msg(dh,IFP_REQ_TYPE,IFP_LS_NEXT,3,0,ctl,sizeof(ctl),TOUT);	// 3=magic !!!???
			ret = ifp_ctl_ret(ctl);
			switch (ret) {
				case 0:
					goto LS_END;
				case 1:
					usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
					unicode2string(strbuf,inbuf);
					strcpy(strbuf2,name);
					if (strcmp(name,"/") != 0) strcat(strbuf2,"/");
					strcat(strbuf2,strbuf); // path/file
					ifp_send_filename(dh,strbuf2);
					usb_control_msg(dh,IFP_REQ_TYPE,IFP_FILE_INFO,1,0,ctl,sizeof(ctl),TOUT);
					if (!ifp_ok(ctl))
						fprintf(stderr, "error\n");
					usb_control_msg(dh,IFP_REQ_TYPE,IFP_GET_FILE_INFO_SIZE,0,0,ctl8,sizeof(ctl8),TOUT);
					//printf("ctl8: %d %d %d %d %d %d %d %d\n",ctl8[0],ctl8[1],ctl8[2],ctl8[3],ctl8[4],ctl8[5],ctl8[6],ctl8[7],ctl8[8]);
					size = ctl8[4] + ctl8[5]*256 + ctl8[6]*256*256 + ctl8[7]*256*256*256;
					if (format)
						printf("-rw-rw-rw-   1 root     root  %d Jan  1 1980 %s/%s\n",size,prefix,strbuf);
					else 
						printf("f %s\t(size %d)\n",strbuf,size);
					usb_control_msg(dh,IFP_REQ_TYPE,IFP_FILE_LEAVE,0,0,ctl,sizeof(ctl),TOUT); //??????
					break;
				case 2:
					usb_bulk_read(dh,IFP_BULK_FROM,inbuf,sizeof(inbuf),TOUT);
					unicode2string(strbuf,inbuf);
					if (format)
						printf("drw-rw-rw-   1 root     root  0 Jan  1 1980 %s/%s\n",prefix,strbuf);
					else printf("d %s\n",strbuf);
					break;
				default:
					fprintf(stderr,"Uknown LS_NEXT return value = %d\n",ret);
					return -3;
			}
    }

LS_END:
  usb_control_msg(dh,IFP_REQ_TYPE,IFP_LS_LEAVE,0,0,ctl,sizeof(ctl),TOUT);
  /* now go through the directories on the stack */
  return 0;
}


int main(int argc, char **argv)
{
  struct stat status;
  struct usb_bus *bus;
  struct usb_bus *bus2 = NULL;
  struct usb_device *tmp_dev;
  struct usb_device *dev = NULL;
  usb_dev_handle *dh;
  int i = 0;
	int c = 2;
  
  usb_init();
  usb_find_busses();
  if (usb_find_devices() <= 0) {
    fprintf(stderr, "No usb devices found.\n");
    return -1;
  }

  for (bus = usb_busses; bus; bus = bus->next) {
    for (tmp_dev = bus->devices; tmp_dev; tmp_dev = tmp_dev->next) {
      if ((tmp_dev->descriptor.idVendor == iRiver_Vendor) &&
        (tmp_dev->descriptor.idProduct == iFP_Product)) {
        dev = tmp_dev;
        goto IFPFOUND;
      }
    }
  }

IFPFOUND:
  if (dev) {
    fprintf(stderr, "IFP Found\n");
    dh = usb_open(dev);
		fprintf(stderr, "Battery %d \n", ifp_battery_condition(dh));
    if (dh && ifp_check_connect(dh)) {
			fprintf(stderr, "IFP COrrectly initialized\n");
      //new_list_dir(dh, "\\My", 0, 0);
      ifp_list_dir_printf2(dh, argv[1], 0);
			usb_close(dh);
    } else {
      if (dh) usb_close(dh);
      fprintf(stderr, "Unable to connect to IFP");
      return -1;
    }
  }
  return 0;
}

